/*
 * Created on Apr 17, 2006
 * 
 * @author jbergin
 *
 */
package googledisplay;

import java.awt.HeadlessException;
import javax.swing.JEditorPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.IOException;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import java.awt.Dimension;
import java.util.Stack;
import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import javax.swing.JButton;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/** This class represents an absolutely minimal "Browser". It will display a page and navigate to links (html pages only)
 * but it has no status line, or any other refinement. It will display text and the raw text of pdf files. 
 * You may be able to copy text from the pages to other applications. It has back and forward buttons to aid navigation. 
 * <p>
 * Note that the html rendering is primitive. It will handle simple html. 
 * <p>
 * Learn more at <a href = http://java.sun.com/docs/books/tutorial/uiswing/components/editorpane.html> http://java.sun.com/docs/books/tutorial/uiswing/components/editorpane.html</a>
 * 
 * @author jbergin
 *
 */
public class GoogleDisplayFrame extends JFrame
{

    /** Create a "Browser" to show a particular page
     * @param pageToDisplay the html page to display. Should be a well formed URL.
     * @throws HeadlessException
     */
    public GoogleDisplayFrame(String pageToDisplay) throws HeadlessException
    {
        this("Google Search Display", pageToDisplay);
    }

    private JEditorPane editor;

    /** Create a browser with a given page title to display a particular page
     * @param pageTitle The window title
     * @param pageToDisplay the html page to display. Should be a well formed URL.
     */
    public GoogleDisplayFrame(String pageTitle, String pageToDisplay)
    {
        super(pageTitle);
        editor = new JEditorPane(); // this does all the work.

        // Let the editor follow hyperlinks
        editor.setEditable(false);
        editor.addHyperlinkListener(new HyperlinkListener()
        {
            public void hyperlinkUpdate(HyperlinkEvent e)
            {
                if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
                {
                    showPage((e.getURL()), true);
                }
            }
        });
        
        // Tell the editor about the page we want
        try
        { 
            showPage(new URL(pageToDisplay), true);
        }
        catch(MalformedURLException m)
        {
            String reply = "The result URL is malformed or missing: " + pageToDisplay;
            System.out.println(reply);
            editor.setText(reply);
        }
        
        // Wrap the editor in a scroller
        JScrollPane editorScrollPane = new JScrollPane(editor);
        editorScrollPane.setVerticalScrollBarPolicy(
                        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        Dimension size = new Dimension(640, 480);
        editorScrollPane.setPreferredSize(size);
        editorScrollPane.setMinimumSize(new Dimension(100, 100));
        
        JPanel control = new JPanel();
        control.setLayout(new FlowLayout());
        JPanel display = new JPanel();
        display.setLayout((new BorderLayout()));
        display.add(editorScrollPane,BorderLayout.CENTER);
        display.add(control,BorderLayout.NORTH);
        
        control.add(back, FlowLayout.LEFT);
        control.add(forward);
        
        back.addActionListener(new ActionListener() 
        {
           public void actionPerformed(ActionEvent e)
           {
               back();
           }
        });
        
        forward.addActionListener(new ActionListener() 
        {
            public void actionPerformed(ActionEvent e)
            {
                forward();
            }           
        });

        // Make the JFrame work with this scroller
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        setContentPane(display);
        setSize(size);
        setMinimumSize(new Dimension(100, 100));
        pack();
        setVisible(true);       
    }
    
    private void showPage (URL url, boolean clearForward) 
    {
        try
        {
            editor.setPage(url);
            backwardStack.push(url);
            if(clearForward)
            {
                forwardStack.clear();
            }
            enableDisable();
        }
        catch (IOException e)
        {
            String reply = "Page could not be read from \"" + url + "\".";
            System.out.println(reply);
            editor.setText(reply);
        }
    }
    
    private void back()
    {
        if(backwardStack.size() > 1)
        {
            URL current = backwardStack.pop();
            forwardStack.push(current);
            URL previous = backwardStack.pop();
            showPage(previous, false);
        }
    }
    
    private void forward()
    {
        if(!forwardStack.empty())
        {
            URL next = forwardStack.pop();
            showPage(next, false);
        }
    }
    
    private void enableDisable()
    {
        back.setEnabled(backwardStack.size() > 1);
        forward.setEnabled(!forwardStack.empty());
    }
    
//    public static void main(String [] args)
//    {
//        new GoogleDisplayFrame("http://csis.pace.edu/~bergin");
//    }
    
    private final Stack<URL> forwardStack = new Stack<URL>();
    private final Stack<URL> backwardStack = new Stack<URL>(); // includes the current page
    private final JButton forward = new JButton("Forward");
    private final JButton back = new JButton("Back");

}
