package googlesearch;

import java.util.List;

import com.google.soap.search.GoogleSearch;
import com.google.soap.search.GoogleSearchFault;

/** Encapsulates a Google search query. It will let you search the Google indexes from a program
 * just as you can from Google.com
 * <p>
 * This is version 2. It has the following differences from V1. (a) it uses Java 5 for its 
 * collections (templates). (b) it constructs the query string as late as possible, using a 
 * recursive look at the decorator chain that defines it. As such, it removes some responsibility
 * from the user to construct a correct query. For exampmle it will guarantee that site: searches
 * put the site: attribute last in any query. 
 * 
 * To use this you will need to get your own Google Key by registering at
 * http://www.google.com/apis/
 * You will also need to download the googleapi.jar file from there and put it into your project. 
 * 
 * <p>
 * Created on Apr 12, 2006
 * 
 * @author jbergin
 *
 */
public abstract class Query
{

    /**
     * Search for the highest ranked results. Up to ten result pages are
     * returned.
     * 
     * @return The search results object with up to ten result pages
     */
    public SearchResult search()
    {
        return search(0);
    }

    /**
     * Return up to ten search result pages starting with the index (ranking)
     * you specify.
     * 
     * @param start
     *            the (zero based) index at which to start
     * @return the search results object with up to ten pages
     */
    public SearchResult search(int start)
    {
        GoogleSearch search = new GoogleSearch();
        search.setKey(Query.getQueryKey());
        search.setStartResult(start);
        search.setQueryString(getSearchString());
        try
        {
            return new SearchResult(search.doSearch());
        }
        catch (GoogleSearchFault f)
        {
            System.out.println(f);
            return null;
        }
    }

    /**
     * Return the top ranked page for this result
     * 
     * @return the top ranked page.
     */
    public SearchResultPage lucky()
    {
        SearchResult all = search();
        List results = all.getResultPages();
        if(results.size() > 0) return all.getResultPages().get(0);
        return SearchResultPage.NULL_PAGE;
    }

    public String toString()
    {
        return "Google Query: " + getSearchString();
    }

    /**
     * Obtain the search string specified when the object was created.
     * 
     * @return the search string
     */
    public abstract String getSearchString();

    /**
     * Used by getSearchString to build up the query string recursively from the
     * parts. Note that the work is done on the unwinding phase of the
     * recursion. Those query fragments that must appear first or last in a
     * query string defer being added to the string until the end, by putting a
     * reference to themselves intoone of the two lists. These lists are then
     * processed at the end after some verification.
     * 
     * @param start
     *            the list of query objects that must come first in a Google
     *            query
     * @param finish
     *            the list of query objects that must come last in a Google
     *            query. e.g. site:
     * @return the query string
     */
    public abstract String buildSearchString(List<Query> start,
            List<Query> finish);

    /**
     * Used only by queries that must place their query text first or last in
     * the full query string. See SiteQuery, for example.
     * 
     * @return the modifier text that should be pre/post pended to the rest of
     *         the query.
     */
    public String modifier()
    {
        return "";
    }

    /** Ask the Google servers to suggest spellings for a possible search "trial".
     * @param trial the trial string that you think may be misspelled
     * @return the google suggested spellings. Often blank. 
     */
    public static String suggestSpelling(String trial)
    {
        GoogleSearch search = new GoogleSearch();
        search.setKey(Query.getQueryKey());
        try
        {
            String result = search.doSpellingSuggestion(trial);
            if(result == null) { return ""; }
            return result;
        }
        catch (GoogleSearchFault f)
        {
            return "";
        }
    }

    /** Get the Google search key that you set with setQueryKey. 
     * @return the Google-supplied search key. 
     */
    public static String getQueryKey()
    {
        return key;
    }

    /** Set the Google search key. This is supplied by Google when you register for this service. 
     * @param googleKey the Google-supplied search key.
     */
    public static void setQueryKey(String googleKey)
    {
        key = googleKey;
    }

    /**
     * An empty Query. Searching with the query will return a
     * SearchResult.NULL_RESULT instead of a valid SearchResult.
     */
    public static final Query NULL_QUERY = new SimpleQuery("")
    {
        public SearchResult search(int start)
        {
            return SearchResult.NULL_RESULT;
        }

        public SearchResultPage lucky()
        {
            return SearchResultPage.NULL_PAGE;
        }
    };

    private static String key = null;
}
