package googlesearch;


import java.util.List;
import java.util.ArrayList;
import java.net.URL;
import java.net.MalformedURLException;
import com.google.soap.search.GoogleSearchResultElement;
import com.google.soap.search.GoogleSearchResult;

/** Represents the result of doing a Google search. It contains information about the 
 * search, such as the search string, and also the result web pages returned from the 
 * search. This is an adapter class to obtain a simpler (and safer) interface to the
 * official GoogleSearchResult object. In particular it hides setter methods that would be
 * dangerous to use here. It also returns the result objects as a java.util.List, rather than an
 * array. 
 * 
 * @author jbergin
 *
 */
public class SearchResult
{
	/** Create a SearchResult object containg a set of result pages and information about the search. This is an
     * adapter for a class supplied by the Google API.
	 * @param adapted The Google API equivalent object
	 */
	public SearchResult(GoogleSearchResult adapted)
	{
		this.adapted = adapted;
	}
	
	/** Get the query string that was used to retrieve these results
	 * @return the query string
	 */
	public String getSearchQuery()
	{
		return adapted.getSearchQuery();
	}
	
	/** Get the google supplied index ranking of the beginning of this result set
	 * @return the start index (ranking)
	 */
	public int getStartIndex()
	{
		return adapted.getStartIndex();
	}
	
	/** Get the Google-supplied index ranking of the end of this result set
	 * @return the end index (ranking). i.e. the index of the last item returned.
	 */
	public int getEndIndex()
	{
		return adapted.getEndIndex();
	}
	
	// TODO add some additional getter methods for other information provided by a
	// GoogleSearchResult object. Don't add the setter methods, however. Add these getters on a 
	// "just in time" basis, however. Don't add them unless you have an actual use for them. 
	
	public String toString()
	{
		return adapted.toString();
	}
	
	/** Obtain an estimate of the number of hits (pages) in this search. Note that the
	 * estimate is often exact, but need not be. 
	 * @return the number of pages Google estimates result from this search
	 */
	public int getEstimatedTotalResultsCount()
	{
		return adapted.getEstimatedTotalResultsCount();
	}
	
	/** Return up to ten result pages in a list. 
	 * @return a list of the result pages
	 */
	public List<SearchResultPage> getResultPages()
	{
		GoogleSearchResultElement [] elements = adapted.getResultElements();
		ArrayList<SearchResultPage> results = new ArrayList<SearchResultPage>(elements.length);
		for(int i = 0; i < elements.length; ++i)
		{	GoogleSearchResultElement result = elements[i];
			String urlString = result.getURL();
			try
			{
				new URL(urlString);// make sure the URL is valid
				results.add(new SearchResultPage(result));				
			}
			catch(MalformedURLException m)
			{
				//nothing. Don't add it to the list.
			}
		}
		return results;
	}
	
	private final GoogleSearchResult adapted;
	
	/** Represents an empty search result. i.e. one generated from a null query
	 * 
	 */
	public static final SearchResult NULL_RESULT = new SearchResult(null)
	{
		public List<SearchResultPage> getResultPages()
		{
			return new ArrayList<SearchResultPage>();
		}
		
		public String getSearchQuery() { return "";}
		
		public  int getEstimatedTotalResultsCount() { return 0; }
		
		public int getStartIndex() { return 0;}
		
		public int getEndIndex() { return 0;} 
		
		public String toString() { return "NULL_RESULT";}
	};
}
