package googlesearch;

import java.net.URL;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;

import com.google.soap.search.GoogleSearch;
import com.google.soap.search.GoogleSearchResultElement;
import com.google.soap.search.GoogleSearchFault;
import java.io.StringWriter;
import googledisplay.GoogleDisplayFrame;

/**
 * Represents one of many pages returned by a Google search. It is essentially
 * just an adapter for a GoogleSearchResultElement object, providing a more
 * convenient interface.
 * 
 * @author jbergin
 * 
 */
public class SearchResultPage
{
    /** Create an object representing a single result page from a search. It contains the URL and other information. It
     * also knows how to image a page in a window. 
     * @param adapted the Google API equivalent object
     */
    public SearchResultPage(GoogleSearchResultElement adapted)
    {
        this.adapted = adapted;
    }

    /**
     * Obtain a URL object for this page
     * 
     * @return a URL
     * @throws RuntimeException
     */
    public URL getURL() throws RuntimeException
    {
        try
        {
            return new URL(adapted.getURL());
        }
        catch (MalformedURLException m)
        {
            throw new RuntimeException(
                    "Saw a malformed URL in SearchResultPage.");
            // Should never happen. We have excluded these from the results.
        }
    }

    /**
     * Retrieve the full URL string including protocol.This string can be pasted
     * into a browser to retrieve the page.
     * 
     * @return the URL string
     */
    public String getURLString()
    {
        return adapted.getURL();
    }
    
    public void displayPage()
    {
        GoogleDisplayFrame frame = new GoogleDisplayFrame(getURLString(), getURLString());
    }

    /**
     * Get the html or text of a page with Mime type "text/html" or
     * "text/plain". Returns an error string in other cases or if there is an
     * error.
     * 
     * @return The html or an error message
     */
    public String getHTML()
    {
        String result = "Error, could not get html";
        StringWriter writer = new StringWriter();
        String contentType = getContentType();
        if(contentType.equalsIgnoreCase("text/html")
                || contentType.equalsIgnoreCase("text/plain"))
        {
            try
            {
                BufferedReader reader = new BufferedReader(
                        new InputStreamReader((InputStream) getContent()));
                while (true)
                {
                    String line = reader.readLine();
                    if(line == null)
                    {
                        break;
                    }
                    writer.write(line);
                    writer.write("\r\n");
                }
                result = writer.toString();
            }
            catch (IOException e)
            {
                // nothing
            }
        }
        else
        {
            result = "Mime type is not html or plain text";
        }
        return result;
    }

    public String getCachedPage()
    {
        GoogleSearch search = new GoogleSearch();
        search.setKey(Query.getQueryKey());
        try
        {
            return new String(search.doGetCachedPage(getURLString()));
        }
        catch (GoogleSearchFault f)
        {
            return "";
        }
    }

    /**
     * Return a small part of the page as a string
     * 
     * @return the page snippet
     */
    public String getSnippet()
    {
        return adapted.getSnippet();
    }

    /**
     * Get the contents of the url of this page. See the JavaDocs for
     * java.net.URL for more information on interpreting the returned result.
     * The objects returned are often InputStreams from which you can read the
     * content. This is the case for an html page.
     * 
     * @return An object containing the page. It may be html, pdf, or other
     * @throws IOException
     */
    public Object getContent() throws IOException
    {

        return getURL().getContent();
    }

    /**
     * Retrieve the Mime type of the page. Common types are "text/html",
     * "text/plain" and "application/pdf". The Mime type tells you how to
     * interpret the content of the page.
     * 
     * @return the Mime type
     */
    public String getContentType()
    {
        try
        {
            return getURL().openConnection().getContentType();
        }
        catch (IOException e)
        {
            return "error: unknown type";
        }
    }

    // TODO add some additional getter methods to match those provided by the
    // GoogleSearchPageELement
    // class. Do not add setters, however. Add these getters on a "just in time"
    // basis, however. Don't
    // add them unless you have an actual use for them.

    public String toString()
    {
        return adapted.toString();
    }

    private final GoogleSearchResultElement adapted;

    /**
     * Represents an empty page.
     * 
     */
    public static final SearchResultPage NULL_PAGE = new SearchResultPage(null)
    {
        public URL getURL()
        {
            throw new RuntimeException("SearchResultPage.NULL_PAGE");
        }

        public String getURLString()
        {
            return "";
        }

        public String getHTML()
        {
            return "";
        }

        public String getContentType()
        {
            return "";
        }

        public Object getContent()
        {
            return null;
        }

        public String getSnippet()
        {
            return "";
        }

        public String toString()
        {
            return "SearchResultPage.NULL_PAGE";
        }
    };
}
